﻿using System;
using System.Windows;
using System.Windows.Data;
using System.Windows.Threading;
using personalplaner.business;

namespace personalplaner.common.mvvm.binding
{
    internal static class FocusController
    {
        #region FocusableProperty

        internal static DependencyProperty GetFocusableProperty(DependencyObject obj)
        {
            return (DependencyProperty)obj.GetValue(FocusablePropertyProperty);
        }

        internal static void SetFocusableProperty(DependencyObject obj, DependencyProperty value)
        {
            obj.SetValue(FocusablePropertyProperty, value);
        }

        internal static readonly DependencyProperty FocusablePropertyProperty =
            DependencyProperty.RegisterAttached(
            "FocusableProperty",
            typeof(DependencyProperty),
            typeof(FocusController),
            new UIPropertyMetadata(null, OnFocusablePropertyChanged));

        static void OnFocusablePropertyChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var element = sender as FrameworkElement;
            if (element == null)
                return;

            var property = e.NewValue as DependencyProperty;
            if (property == null)
                return;

            element.DataContextChanged += delegate
            {
                CreateHandler(element, property);
            };

            if (element.DataContext != null)
            {
                CreateHandler(element, property);
            }
        }

        static void CreateHandler(DependencyObject element, DependencyProperty property)
        {
            var focusMover = element.GetValue(FrameworkElement.DataContextProperty) as IFocusMover;

            if (focusMover == null)
            {
                var parent = System.Windows.Media.VisualTreeHelper.GetParent(element);
                while (parent != null)
                {
                    focusMover = parent.GetValue(FrameworkElement.DataContextProperty) as IFocusMover;
                    if (focusMover != null)
                        break;

					parent = LogicalTreeHelper.GetParent(parent);
                }
            }


            if (focusMover == null)
            {
                var handler = element.GetValue(MoveFocusSinkProperty) as MoveFocusSink;
                if (handler != null)
                {
                    handler.ReleaseReferences();
                    element.ClearValue(MoveFocusSinkProperty);
                }
            }
            else
            {
                var handler = new MoveFocusSink(element as UIElement, property);
                focusMover.MoveFocus += handler.HandleMoveFocus;
                element.SetValue(MoveFocusSinkProperty, handler);
            }
        }

        #endregion // FocusableProperty

        #region MoveFocusSink

        static public UIElement FocusedElement
        {
            get { return (_focusedElement == null || !_focusedElement.IsAlive) ? null : _focusedElement.Target as UIElement; }
            set
            {
                if (value == null)
                    _focusedElement = null;
                else
                    _focusedElement = new WeakReference(value);
            }
        }
        static WeakReference _focusedElement;

        static readonly DependencyProperty MoveFocusSinkProperty = DependencyProperty.RegisterAttached(
            "MoveFocusSink",
            typeof(MoveFocusSink),
            typeof(FocusController),
            new UIPropertyMetadata(null));

        private class MoveFocusSink
        {
            public MoveFocusSink(UIElement element, DependencyProperty property)
            {
                Element = element;
                _property = property;
            }

            internal void HandleMoveFocus(object sender, MoveFocusEventArgs e)
            {
                UIElement element = Element;

                if (element == null || _property == null)
                    return;

                var binding = BindingOperations.GetBinding(element, _property);
                if (binding == null)
                    return;

                if (e.FocusedProperty != binding.Path.Path)
                    return;

				if (!element.IsEnabled)
					return;

                // Delay the call to allow the current batch
                // of processing to finish before we shift focus.
                element.Dispatcher.BeginInvoke((Action)delegate
                {
					if (element != null)
						element.Focus();
                },
				DispatcherPriority.Normal);//.Normal);//.Background);
				e.Focused = true;

                FocusController.FocusedElement = element;
            }

            internal void ReleaseReferences()
            {
                Element = null;
                _property = null;
            }

            public UIElement Element
            {
                get { return (_element == null || !_element.IsAlive) ? null : _element.Target as UIElement; }
                set
                {
                    if (value == null)
                        _element = null;
                    else
                        _element = new WeakReference(value);
                }
            }
            WeakReference _element;

            DependencyProperty _property;
        }

        #endregion // MoveFocusSink
    } 
}